#include <fcntl.h>
#include <sys/mman.h>
#include "rtc.h"

#ifndef __SW_3
#if defined(__SW_3R) || defined(__SW_3S)
#define __SW_3
#endif
#endif


#define TRUE	1
#define FALSE	0

#define AMPM			0x40	/* clock in 12 hour AM/PM format */
#define MILTIME			0x00	/* clock in 24 hour format */
#define TRANSFER_ENABLE	0x80	/* enable transfer counters -> registers */
#define CMD_REG			(*(clk_reg+11))

/*
 * Clock setup for DS1386 part. This is an embedded part so we can use
 * what ever conventions we want for the century byte.

 * addresses that we use (all fields are BCD)

 0 hundreths of seconds
 1 seconds
 2 minutes
 4 hours & hour format
 6 day of week (we set this up but do not use it)
 8 day
 9 month
 A year (0-99)
 B command register 
 E user area 1 (century)
 F user area 2 (OS reserved)
 10 - 7FFF (7FFF) user area 32K (8K) battery backed up SRAM

 */
#if defined (__SW_3) && !defined(__QNXNTO__)
/* 32 bit versions */
void *make_clk_pntr(unsigned base) {
	static int	fd = -1;
	void		*addr;
	
	if (fd == -1) {
		if ((fd = shm_open ("Physical", O_RDWR, 0777)) == -1) {
			fprintf(stderr, "Unable to open physical memory (%s)", strerror (errno));
			exit(-1);
		}
	}
	addr = mmap (0, 8196, PROT_READ|PROT_WRITE|PROT_NOCACHE, MAP_SHARED, fd, base);
	if (addr == (void *) -1) {
		fprintf( stderr, "unable to map physical memory (%s)\n", strerror (errno));
		exit(-1);
	}
	return addr;
}

void free_clk_pntr (void *phys) {
	munmap (phys, 8196);
}

#endif

#ifdef __SW_2
/* 16 bit version */

#include <sys/seginfo.h>
static unsigned sel;

void far *make_clk_pntr(unsigned base) {
	void far *addr;
	
	if((sel = qnx_segment_overlay(base, 8196)) == -1) {
		fprintf(stderr, "Unable get overlay segment(%s)", strerror (errno));
		exit(-1);
	}

	addr = MK_FP(sel, 0);
	return addr;
}

void free_clk_pntr (void *phys) {
	phys = phys;
	qnx_segment_free(sel);
}
#endif

#ifdef __QNXNTO__
/* NTO version */
#include <sys/mman.h>

void *make_clk_pntr(unsigned base) {
	void *addr;
	
	if((addr = mmap(0, 8196, PROT_READ|PROT_WRITE, MAP_PHYS|MAP_SHARED, NOFD, base)) == MAP_FAILED) {
		fprintf(stderr, "Unable mmap memory(%s)", strerror (errno));
		exit(-1);
	}
	return addr;
}

void free_clk_pntr (void *phys) {
	munmap(phys, 8196);
}
#endif
 
extern	unsigned	date[2], cent, year, month, day, hours, minutes, seconds;
extern	unsigned	msec, dow;
extern	unsigned	int_bcd(), bcd_int();

ds1386(int setflag, unsigned base) 
{
	struct timespec cur;
	
	volatile char *clk_reg = (char *) make_clk_pntr(base);

	if(setflag) { 			/* Set hardware clock.	*/

#ifndef __QNXNTO__
		getclock(TIMEOFDAY, &cur);
#else
		clock_gettime(CLOCK_REALTIME,&cur);
#endif

		msec = cur.tv_nsec/1000000;

		if ( year > 99 ) year -= 100;
		msec = msec/10;					/* clock only stores hundreths */

		// convert binary to BCD
		msec	= int_bcd(msec);
		seconds	= int_bcd(seconds);
		minutes	= int_bcd(minutes);
		hours	= int_bcd(hours);
		day 	= int_bcd(day);
		month	= int_bcd(month);
		year	= int_bcd(year);
		cent	= int_bcd(cent);

#ifdef DIAG
		fprintf(stderr,"rtc set: cent=%d; year=%d (after adjustment)\n",cent,year);
#endif

		month |= *(clk_reg+9) & 0x40;	/* ESQW# */

		*(clk_reg+9)  = 0;	/* turn off clock */

		*(clk_reg+0)  = msec;
		*(clk_reg+1)  = seconds;
		*(clk_reg+2)  = minutes;
		*(clk_reg+4)  = hours | MILTIME;
		*(clk_reg+6)  = dow;
		*(clk_reg+8)  = day;
		*(clk_reg+10) = year;
		*(clk_reg+14) = cent;

		*(clk_reg+9)  = month & 0x5f;	/* EOSC# low to start clock */
		CMD_REG = CMD_REG | TRANSFER_ENABLE ;

	}else {					/* Get the time from the hardware clock.	*/

		if(!(CMD_REG & TRANSFER_ENABLE)) {
fprintf(stderr, "Transfer Enable not set!\n");
			CMD_REG = CMD_REG | TRANSFER_ENABLE ;
			cur.tv_sec = 0;
			cur.tv_nsec = 10000000;
			nanosleep(&cur, NULL); 	/* wait 0.01 sec for update */
		}

		CMD_REG = CMD_REG & !TRANSFER_ENABLE; 	/* disable transfers */
		
		// convert BCD to binary 
		msec	= bcd_int(*(clk_reg+0)) * 10;	// seconds
		seconds = bcd_int(*(clk_reg+1));		// seconds
		minutes = bcd_int(*(clk_reg+2));		// minutes
		hours	= bcd_int(*(clk_reg+4) & 0x3f);	// hours
		day		= bcd_int(*(clk_reg+8));		// day
		month	= bcd_int(*(clk_reg+9) & 0x3f);	// month
		year	= bcd_int(*(clk_reg+10));		// year
		cent	= bcd_int(*(clk_reg+14));		// century

		CMD_REG = CMD_REG | TRANSFER_ENABLE ;

		if ( cent == 20 ) year += 100;
#ifdef DIAG
		fprintf(stderr,"rtc read: cent=%d; year=%d\n",cent,year);
#endif
	} 

	free_clk_pntr ((void *) clk_reg);

	return(0);
}
