/*
 * this is the memory mapped version which deals with the
 * FOX RTC-72423 part. A version also exists if the part
 * is io mapped
 */

#include <fcntl.h>
#include <sys/mman.h>
#include "rtc.h"

#define TRUE	1
#define FALSE	0

/* register D */
#define SECADJ			0x80	/* 30 sec adj clock */
#define IRQFLAG			0x40	/* IRQ reset/flag */
#define BUSY			0x20	/* clock in transferring data */
#define HOLD			0x10	/* stop clock transfer */

/* register E */
#define T0				0x80	/* interrupt source */
#define T1				0x40	/* interrupt source */
#define ITRPT			0x20	/* interrupt mode */
#define MASK			0x10	/* interrupt mask */

/* register F */
#define TEST			0x80	/* 30 sec adj clock */
#define AMPM			0x40	/* clock in 12 hour AM/PM format */
#define STOP			0x20	/* stop the clock divider */
#define RESET			0x10	/* reset the clock chip register */

#define REG_D			(base+0xd)
#define REG_E			(base+0xe)
#define REG_F			(base+0xf)

/*
 * Clock setup for FOX RTC-72423 part. This is an embedded part so we can use
 * what ever conventions we want for the year regiusters. Note there is no
 * Century byte or other ram on this so we use the year between 0-50 to
 * indicate if it is the next century/
 *
 * addresses that we use (all fields are BCD)

 0 seconds 1
 1 seconds 10
 2 minutes 1
 3 minutes 10
 4 hours 1 
 5 hours 10 & hour format
 6 day 1
 7 day 10
 8 month 1
 9 month 10
 A year 1
 B year 10
 C day of week (we set this up but do not use it)
 D command register 
 E interrup control
 F command register 

 */

void *make_clk_pntr(unsigned base);
void free_clk_pntr (void *phys);
 
extern	unsigned	date[2], cent, year, month, day, hours, minutes, seconds;
extern	unsigned	dow;
extern	unsigned	int_bcd(), bcd_int();

rtc72423(int setflag, unsigned base) 
{
	struct timespec cur;
	
	if(setflag) { 			/* Set hardware clock.	*/

		if ( year > 99 ) year -= 100;

		// convert binary to BCD
		seconds	= int_bcd(seconds);
		minutes	= int_bcd(minutes);
		hours	= int_bcd(hours);
		day 	= int_bcd(day);
		month	= int_bcd(month);
		year	= int_bcd(year);
		cent	= int_bcd(cent);

        // we assume the chip has never been setup.
		outp(REG_F, RESET | AMPM | STOP); /* test=0 */

		outp(base+0, seconds);
		outp(base+1, seconds>>4);
		outp(base+2, minutes);
		outp(base+3, minutes>>4);
		outp(base+4, hours);
		outp(base+5, hours>>4);
		outp(base+6, day);
		outp(base+7, day>>4);
		outp(base+8, month);
		outp(base+9, month>>4);
		outp(base+10, year);
		outp(base+11, year>>4);
		outp(base+6, dow);

		outp(REG_E, 0);				/* Enable signal out @ 1/64 sec */

		outp(REG_D, 0);				/* 30sec=0, IRQ=0, HOLD=0 */

		outp(REG_F, AMPM);			/* start clock. TEST=0, STOP=0, RESET=0 */

	}else {						/* Get the time from the hardware clock.	*/

		outp(REG_D, HOLD);			/* stop the clock */

		while(inp(REG_D) & BUSY) {
			outp(REG_D, 0);				/* HOLD=0 */
			cur.tv_sec = 0;
			cur.tv_nsec = 190000;
			nanosleep(&cur, NULL); 	/* wait 190 usec for update */
			outp(REG_D, HOLD);			/* stop the clock */
		}

		// get the data
		seconds	= (inp(base+1)  <<4) | inp(base+0); 
		minutes	= (inp(base+3)  <<4) | inp(base+2); 
		hours	= ((inp(base+5) & 3)  <<4) | inp(base+4);
		day		= (inp(base+7)  <<4) | inp(base+6); 
		month	= (inp(base+9)  <<4) | inp(base+8); 
		year	= (inp(base+11) <<4) | inp(base+10); 

		outp(REG_D, 0);				/* HOLD=0 */

		// convert BCD to binary 
		seconds = bcd_int(seconds);
		minutes = bcd_int(minutes);
		hours	= bcd_int(hours);
		day		= bcd_int(day);
		month	= bcd_int(month);
		year	= bcd_int(year);

		if ( year < 50 ) year += 100;
	} 

	return(0);
}
